//
//  DetailViewController.m
//  iCloud Reports
//
//  Created by Sams Publishing on 2/4/13.
//  Copyright (c) 2013 Champlain Arts. All rights reserved.
//

#import "DetailViewController.h"

@interface DetailViewController ()

@property (strong, nonatomic) UIPopoverController *masterPopoverController;
@property (strong, nonatomic) ReportDocument *document;                   //1

@end

@implementation DetailViewController

#pragma mark - Managing the detail item

- (void)setDetailItem:(id)newDetailItem                                   //2
{
  if (_detailItem != newDetailItem) {
    _detailItem = newDetailItem;
  
  if (_document) {
    [_document closeWithCompletionHandler: nil];
  }
    
  _document = [[ReportDocument alloc] initWithFileURL:self.detailItem]; //3
  _document.delegate = self;
  
    
  NSFileManager *fm = [NSFileManager defaultManager];                   //4
  if ([fm fileExistsAtPath: [self.detailItem path]])
    [_document openWithCompletionHandler:nil];
  else
    [_document saveToURL:self.detailItem forSaveOperation:UIDocumentSaveForCreating completionHandler:nil];
  }
}

- (void)viewWillAppear:(BOOL)animated {                                     //5
  [super viewWillAppear:animated];
  
  self.textView.text = @"";
  
   if (self.detailItem) {
    _document = [[ReportDocument alloc] initWithFileURL:self.detailItem];
    _document.delegate = self;
    
    // open or create
    NSFileManager *fm = [NSFileManager defaultManager];
    if ([fm fileExistsAtPath: [self.detailItem path]])
      [_document openWithCompletionHandler:^(BOOL success) {                //6
        if (!success) {
          NSLog(@"ERROR: cannot open %@", _document.fileURL);
        }
      }];
    else
      [_document saveToURL:self.detailItem forSaveOperation:UIDocumentSaveForCreating completionHandler:nil];
  }
  
  // register for keyboard notifications                                    //7
  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector (keyboardWillShow:) name:UIKeyboardWillShowNotification object:nil];
  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector (keyboardWillHide:) name:UIKeyboardWillHideNotification object:nil];

}

- (void)viewWillDisappear:(BOOL)animated {                                   //8
  [super viewWillDisappear: animated];
  
  NSString *newText = self.textView.text;
  
  _document.documentText = newText;

  [_document closeWithCompletionHandler: ^(BOOL success) {                    //9
    
    if (!success) {
      NSLog(@"ERROR: Can't close %@", _document.fileURL);
    }
  }];             
  
  [[NSNotificationCenter defaultCenter] removeObserver:self name:UIKeyboardWillShowNotification object:nil];                                                                         //10
  [[NSNotificationCenter defaultCenter] removeObserver:self name:UIKeyboardWillHideNotification object:nil];
  
}

- (void)didReceiveMemoryWarning
{
  [super didReceiveMemoryWarning];
  // Dispose of any resources that can be recreated.
}

- (void)keyboardWillShow: (NSNotification *)aNotification {
  NSDictionary *info = [aNotification userInfo];
  CGRect kbSize = [[info objectForKey: UIKeyboardFrameEndUserInfoKey] CGRectValue];
  double duration = [[info objectForKey: UIKeyboardAnimationDurationUserInfoKey] doubleValue];
  UIEdgeInsets insets = self.textView.contentInset;
  insets.bottom += kbSize.size.height;
  [UIView animateWithDuration: duration animations:^ {
    self.textView.contentInset = insets;
  }];
}

- (void)keyboardWillHide: (NSNotification *)aNotification {
  NSDictionary *info = [aNotification userInfo];
  double duration = [[info objectForKey: UIKeyboardAnimationDurationUserInfoKey] doubleValue];

  UIEdgeInsets insets = self.textView.contentInset;
  insets.bottom = 0;
  [UIView animateWithDuration: duration animations:^ {
    self.textView.contentInset = insets;
  }];
}

#pragma mark - ReportDocumentDelegate protocol
- (void)documentContentsDidChange:(ReportDocument *)document {
  dispatch_async(dispatch_get_main_queue(), ^ {
    self.textView.text = document.documentText;
    //[_document updateChangeCount: UIDocumentChangeDone];
  });
  
}

#pragma mark - Split view

- (void)splitViewController:(UISplitViewController *)splitController willHideViewController:(UIViewController *)viewController withBarButtonItem:(UIBarButtonItem *)barButtonItem forPopoverController:(UIPopoverController *)popoverController
{
    barButtonItem.title = NSLocalizedString(@"Master", @"Master");
    [self.navigationItem setLeftBarButtonItem:barButtonItem animated:YES];
    self.masterPopoverController = popoverController;
}

- (void)splitViewController:(UISplitViewController *)splitController willShowViewController:(UIViewController *)viewController invalidatingBarButtonItem:(UIBarButtonItem *)barButtonItem
{
    // Called when the view is shown again in the split view, invalidating the button and popover controller.
    [self.navigationItem setLeftBarButtonItem:nil animated:YES];
    self.masterPopoverController = nil;
}

@end
