//
//  MasterViewController.m
//  iCloud Reports
//
//  Created by Sams Publishing on 2/4/13.
//  Copyright (c) 2013 Champlain Arts. All rights reserved.
//

#import "MasterViewController.h"
#import "Constants.h"
#import "DetailViewController.h"
#import "ReportDocument.h" //jf
#import "AppDelegate.h"
#import "SharediCloudController.h"

@interface MasterViewController ()

@property (weak,nonatomic) SharediCloudController* iCloudController;           //1

@end

@implementation MasterViewController

- (id)initWithCoder:(NSCoder*)aDecoder
{
  if(self = [super initWithCoder:aDecoder])
    {
      AppDelegate *delegate = (AppDelegate*)[[UIApplication sharedApplication] delegate];
      _iCloudController = [delegate iCloudController];                         //2
    }
  
  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector (reloadData) name:SharediCloudControllerDidChangeDataNotification object:nil];
  
  return self;
}

- (void)awakeFromNib                                                           //3
{
  [super awakeFromNib];
  
  self.navigationItem.leftBarButtonItem = self.editButtonItem;
  
  [self.tableView reloadData];                                                 //4
	
}

- (void)reloadData {
  [self.tableView reloadData];                                                 //5
}

- (void)viewDidLoad
{
  [super viewDidLoad];
  
	// Do any additional setup after loading the view, typically from a nib.
  self.navigationItem.leftBarButtonItem = self.editButtonItem;

  self.detailViewController = (DetailViewController *)[[self.splitViewController.viewControllers lastObject] topViewController];
  
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (NSString *)newUntitledDocumentName {
  return [_iCloudController newUntitledDocumentName];
}

- (IBAction)addDocument: (id)sender {
  
  self.addButton.enabled = NO;                                               //1
  
  dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0),
                 ^{                                                                       //2
                   NSFileManager *fm = [NSFileManager defaultManager];                      //3
                   NSURL *newDocumentURL = [fm URLForUbiquityContainerIdentifier:nil];
                   newDocumentURL = [newDocumentURL
                                     URLByAppendingPathComponent:DocumentsDirectoryName isDirectory:YES];
                   newDocumentURL = [newDocumentURL URLByAppendingPathComponent:
                                     [self newUntitledDocumentName]];
                   
                   dispatch_async (dispatch_get_main_queue(),
                                   ^{                           //3
                                   // update documents
                                   [_iCloudController addDocument: newDocumentURL];
                                   
                                   // update UI                                                        //4
                                   NSIndexPath *newCellIndexPath = [NSIndexPath
                                                                    indexPathForRow:([_iCloudController numberOfDocuments] - 1) inSection:0];
                                   [self.tableView insertRowsAtIndexPaths:
                                    [NSArray arrayWithObject:newCellIndexPath]
                                                         withRowAnimation: UITableViewRowAnimationAutomatic];
                                   [self.tableView selectRowAtIndexPath: newCellIndexPath animated:YES
                                                         scrollPosition:UITableViewScrollPositionMiddle];
                                   
                                   UITableViewCell *selectedCell = [self.tableView
                                                                    cellForRowAtIndexPath: newCellIndexPath];
                                   if ([[UIDevice currentDevice] userInterfaceIdiom] ==
                                       UIUserInterfaceIdiomPad) { //5
                                                                  //set selectedItem for iPad
                                                                  // find document
                                     NSIndexPath *cellPath = [self.tableView indexPathForSelectedRow];
                                     NSURL *theURL = [_iCloudController documentURLAt:[cellPath row]];
                                     _detailViewController.detailItem = theURL;
                                   } else if ([[UIDevice currentDevice] userInterfaceIdiom] ==
                                              UIUserInterfaceIdiomPhone) {
                                     //start segue for iPhone
                                     [self performSegueWithIdentifier: DisplayDetailSegue 
                                                               sender: selectedCell];   //6
                                   };
                                   
                                   //re-enable add button
                                   self.addButton.enabled = YES;
                                   
                                   }); //main queue
                 }); //background queue
}

- (void)prepareForSegue: (UIStoryboardSegue*)segue sender:(id)sender {
  if ([segue.identifier isEqualToString: DisplayDetailSegue]) {      //1
    
    DetailViewController *destVC = (DetailViewController *)segue.destinationViewController;  //2
    
    NSIndexPath *cellPath = [self.tableView indexPathForSelectedRow];
    UITableViewCell *theCell = [self.tableView cellForRowAtIndexPath: cellPath];
    NSURL * theURL = [_iCloudController documentURLAt: [cellPath row]];
    destVC.detailItem = theURL;
    destVC.navigationItem.title = theCell.textLabel.text;             //3
  }
}

#pragma mark - Table View

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
  return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
  return [_iCloudController numberOfDocuments];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
  UITableViewCell *newCell = [tableView dequeueReusableCellWithIdentifier:ReportEntryCell forIndexPath:indexPath];
  
	if (!newCell)
    newCell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:ReportEntryCell];
  
  if (!newCell)
    return nil;
  
  newCell.textLabel.text = [_iCloudController stringForDocumentNameAt: [indexPath row]];
  
  return newCell;
  
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
  if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPad) {
    NSURL * theURL = [_iCloudController documentURLAt: [indexPath row]];      //1
    self.detailViewController.detailItem = theURL;                 	          //2
    self.detailViewController.navigationItem.title = [_iCloudController stringForDocumentNameAt: [indexPath row]];  //3
  }
}
 
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the specified item to be editable.
    return YES;
}

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
  if (editingStyle == UITableViewCellEditingStyleDelete) {                //1
    NSURL *fileURL = [_iCloudController documentURLAt: [indexPath row]];  //2
    
    [_iCloudController deleteDocument:fileURL];                           //3
      
    [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationAutomatic];               //4
  }
}

@end
