//
//  JFDetailViewController.m
//  Placid
//
//  Created by Jesse Feiler on 7/3/13.
//  Copyright (c) 2013 Champlain Arts. All rights reserved.
//

#import "JFDetailViewController.h"

@interface JFDetailViewController ()     //1
@property (strong, nonatomic) UIPopoverController *masterPopoverController;
- (void)configureView;
@end

@implementation JFDetailViewController {
  JFWrappedDocument *_document;
}

-(void)setDetailItem: (NSURL *)newDetailItem
{
  _detailItem = newDetailItem;
  [self configureView ];
}

#pragma mark - Managing the detail item

- (void)configureView
{
  // Update the user interface for the detail item.
    if (self.detailItem) {
    _document = [[JFWrappedDocument alloc ]initWithFileURL: self.detailItem]; //2
    
    _document.delegate = self;
    
    NSFileManager *fm = [NSFileManager defaultManager];
    
    if ([fm fileExistsAtPath: [self.detailItem path]])
      [_document openWithCompletionHandler:nil];
    else
    {
      [_document saveToURL: self.detailItem forSaveOperation: UIDocumentSaveForCreating completionHandler:
       ^(BOOL success) {
         if (success) {
           _textField.text = _document.documentText;
           _locationField.text = _document.documentLocation;
         }
       }];
    }
  }
}

- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view, typically from a nib.
    [self configureView];
}

- (void)viewWillAppear:(BOOL)animated {
  
  [super viewWillAppear:animated];
  
}

- (void)viewWillDisappear:(BOOL)animated {
  
  [super viewWillDisappear:animated];
  
  NSString *newText = self.textField.text;   //3
  NSString *newLocation = self.locationField.text;
  
  _document.documentText = newText;   //4
  _document.documentLocation = newLocation;
  
  NSLog (@" %s %@ %@", __PRETTY_FUNCTION__, newText, newLocation);
 
  [_document closeWithCompletionHandler:nil];  //5
  
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

# pragma mark - JFWrappedDocument delegate

- (void)documentContentsDidChange: (JFWrappedDocument *)document {  //5
  NSLog (@" %s %@", __PRETTY_FUNCTION__, document);
  dispatch_async (dispatch_get_main_queue(), ^{
    self.textField.text = _document.documentText;
    self.locationField.text = _document.documentLocation;
    NSLog (@" %@ %@", self.textField.text, self.locationField.text);
  });
}

#pragma mark - Split view

- (void)splitViewController:(UISplitViewController *)splitController willHideViewController:(UIViewController *)viewController withBarButtonItem:(UIBarButtonItem *)barButtonItem forPopoverController:(UIPopoverController *)popoverController
{
    barButtonItem.title = NSLocalizedString(@"Master", @"Master");
    [self.navigationItem setLeftBarButtonItem:barButtonItem animated:YES];
    self.masterPopoverController = popoverController;
}

- (void)splitViewController:(UISplitViewController *)splitController willShowViewController:(UIViewController *)viewController invalidatingBarButtonItem:(UIBarButtonItem *)barButtonItem
{
    // Called when the view is shown again in the split view, invalidating the button and popover controller.
    [self.navigationItem setLeftBarButtonItem:nil animated:YES];
    self.masterPopoverController = nil;
}

@end
