//
//  JFMasterViewController.m
//  Placid
//
//  Created by Jesse Feiler on 7/3/13.
//  Copyright (c) 2013 Champlain Arts. All rights reserved.
//

#import "JFMasterViewController.h"
#import "JFDetailViewController.h"
#import "JFWrappedDocument.h"
#import "JFAppDelegate.h"

NSString *JFDocFilenameExtension = @"plreport";          //1
NSString *DisplayDetailSegue = @"showDetail";            //2
NSString *JFDocumentsDirectoryName = @"Documents";       //3
NSString *DocumentEntryCell = @"DocumentEntryCell";      //4

@interface JFMasterViewController () {

NSMutableArray *_objects; //template                     //5
NSMutableArray *documents; //TE3                         //6
NSMetadataQuery *_query;
}
@end

@implementation JFMasterViewController

#pragma mark - load and memory

- (void)awakeFromNib
{
  
  [super awakeFromNib];
  
  if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPad) {
    self.clearsSelectionOnViewWillAppear = NO;
    self.preferredContentSize = CGSizeMake(320.0, 600.0);
  }
    
  if (!documents)
    documents = [[NSMutableArray alloc] init];                             //7
  
  self.navigationItem.leftBarButtonItem = self.editButtonItem;             //8
  
  [self setupAndStartQuery];                                               //9
}

- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view, typically from a nib.
    self.navigationItem.leftBarButtonItem = self.editButtonItem;

    UIBarButtonItem* addButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemAdd target:self action:@selector(addDocument:)];                          //10
    self.navigationItem.rightBarButtonItem = addButton;                    //11
    self.detailViewController = (JFDetailViewController *)[[self.splitViewController.viewControllers lastObject] topViewController];
  
  

}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark  - documents

/*
 - (void)insertNewObject:(id)sender
{
    if (!_objects) {
        _objects = [[NSMutableArray alloc] init];
    }
    [_objects insertObject:[NSDate date] atIndex:0];
    NSIndexPath *indexPath = [NSIndexPath indexPathForRow:0 inSection:0];
    [self.tableView insertRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationAutomatic];
}
*/
- (IBAction)addDocument:(id)sender {                                                 //12
  self.addButton.enabled = NO;
  
  dispatch_async (dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{  //13
    
    NSFileManager* fm = [NSFileManager defaultManager];
    NSURL *newDocumentURL = [fm URLForUbiquityContainerIdentifier:nil];              //14
    
    newDocumentURL = [newDocumentURL URLByAppendingPathComponent:JFDocumentsDirectoryName isDirectory:YES];  //15
    newDocumentURL = [newDocumentURL URLByAppendingPathComponent:[self newUntitledDocumentName]];  //16
    
    dispatch_async(dispatch_get_main_queue(), ^{                                     //17
      [documents addObject:newDocumentURL];
      
      //update table                                               //18
      NSIndexPath* newCellIndexPath =
        [NSIndexPath indexPathForRow: ([documents count] - 1) inSection: 0];
      [self.tableView insertRowsAtIndexPaths: [NSArray arrayWithObject: newCellIndexPath]
         withRowAnimation:UITableViewRowAnimationAutomatic];
                   
    [self.tableView selectRowAtIndexPath:newCellIndexPath animated:YES scrollPosition:(UITableViewScrollPositionMiddle)];
      UITableViewCell *selectedCell = [self.tableView cellForRowAtIndexPath: newCellIndexPath];
      [self performSegueWithIdentifier:DisplayDetailSegue sender:selectedCell];
      
      self.addButton.enabled = YES;                                                  //19
    });
  });
}

- (NSString *)newUntitledDocumentName {                                               //20
  
  NSInteger docCount = 1;
  NSString *newDocName = nil;
  
  // Manage document list updates
  BOOL done = NO;
  
  while (!done) {
    newDocName = [NSString stringWithFormat: @"Note %d.%@", docCount, JFDocFilenameExtension];
    
    //search for first unused docCount value
    BOOL nameExists = NO;
    for (NSURL *aURL in documents) {
      if ([[aURL lastPathComponent] isEqualToString: newDocName]) {
        docCount++;
        nameExists = YES;
        break;
      }
    }
    
    if (!nameExists)
      done = YES;
  }
  
  return newDocName;
  
}

#pragma mark - Table View

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
  return 1;                                                                                //21
  //return [documents count];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return documents.count;                                                                //22
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
  UITableViewCell *newCell = [tableView dequeueReusableCellWithIdentifier:DocumentEntryCell forIndexPath:indexPath];
  
  if (!newCell)
    newCell = [[UITableViewCell alloc] initWithStyle: UITableViewCellStyleDefault reuseIdentifier: DocumentEntryCell];

  if (!newCell)
    return nil;
  
  NSURL *fileURL = [documents objectAtIndex:[indexPath row]];                               //23
  
  newCell.textLabel.text = [[fileURL lastPathComponent] stringByDeletingPathExtension];     //24
  
  return newCell;
}

- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the specified item to be editable.
    return YES;
}

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        
      NSURL *fileURL = [documents objectAtIndex: [indexPath row]];                           //25
      
      dispatch_async (dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{      //26
        NSFileCoordinator *fc = [[NSFileCoordinator alloc] initWithFilePresenter:nil];
        [fc coordinateWritingItemAtURL:fileURL options:NSFileCoordinatorWritingForDeleting error:nil byAccessor: ^(NSURL *newURL) {
          NSFileManager *fm = [[NSFileManager alloc] init];
          [fm removeItemAtURL:newURL error:nil];
        }];
      });
      
      [documents removeObjectAtIndex:[indexPath row]];                                        //27
        
      [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath]
        withRowAnimation:UITableViewRowAnimationAutomatic];                                    //28
    }
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
  UITableViewCell *theCell = [self.tableView cellForRowAtIndexPath:indexPath];
  NSURL *theURL = [documents objectAtIndex: [indexPath row]];
  NSLog (@"  %ld %@", (long)[indexPath row], theURL);
  self.detailViewController.detailItem = theURL;
  self.detailViewController.navigationItem.title = theCell.textLabel.text;
  NSLog (@" %@", theCell.textLabel.text);
}


#pragma mark - Segue

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
  
  if (![segue.identifier isEqualToString:@"showDetail"])
      return;
  
  JFDetailViewController *destVC = (JFDetailViewController *)segue.destinationViewController;
  
  NSIndexPath *cellPath = [self.tableView indexPathForSelectedRow];
  UITableViewCell *theCell = [self.tableView cellForRowAtIndexPath: cellPath];
  NSURL *theURL = [documents objectAtIndex: [cellPath row]];                                   //29
  
  destVC.detailItem = theURL;                                                                  //30
  destVC.navigationItem.title = theCell.textLabel.text;                                        //31
  
}

#pragma mark - iCloud query

- (NSMetadataQuery *)wrappedDocumentQuery {                                                     //32
  NSMetadataQuery *aQuery = [[NSMetadataQuery alloc] init];
  if (aQuery) {
    [aQuery setSearchScopes: [NSArray arrayWithObject: NSMetadataQueryUbiquitousDocumentsScope]]; //33
    NSString *filePattern = [NSString stringWithFormat: @"*.%@", JFDocFilenameExtension];
    [aQuery setPredicate: [NSPredicate predicateWithFormat:@"%K LIKE %@", NSMetadataItemFSNameKey, filePattern]];
  }
  
    return aQuery;
}

- (void)setupAndStartQuery {                   //34
  if (!_query)
    _query = [self wrappedDocumentQuery];
  [[NSNotificationCenter defaultCenter] addObserver: self selector:@selector (processFiles) name: NSMetadataQueryDidFinishGatheringNotification object: nil];
  [[NSNotificationCenter defaultCenter] addObserver: self selector:@selector (processFiles) name: NSMetadataQueryDidUpdateNotification object: nil];
  
  [_query startQuery];

}

- (void)processFiles {                      //35
  
  NSMutableArray *discoveredFiles = [NSMutableArray array];   //36
  
  [_query disableUpdates];                         //37
  
  NSArray *queryResults = [_query results];
  for (NSMetadataItem *result in queryResults) {
    NSURL *fileURL = [result valueForAttribute: NSMetadataItemURLKey];
    NSNumber *aBool = nil;
    [fileURL getResourceValue: &aBool forKey: NSURLIsHiddenKey error: nil];
    if (aBool && ![aBool boolValue])         //38
      [discoveredFiles addObject: fileURL];
  }
  
  [documents removeAllObjects];   //39
  [documents addObjectsFromArray: discoveredFiles];
  [self.tableView reloadData];
  
  [_query enableUpdates];     //40
  
}

@end
