/*
     File: JFWrappedDoc.m
 Abstract: The NSDocument subclass for reading/writing it data and connecting with iCloud.
  Version: 1.1
 
 Disclaimer: IMPORTANT:  This Apple software is supplied to you by Apple
 Inc. ("Apple") in consideration of your agreement to the following
 terms, and your use, installation, modification or redistribution of
 this Apple software constitutes acceptance of these terms.  If you do
 not agree with these terms, please do not use, install, modify or
 redistribute this Apple software.
 
 In consideration of your agreement to abide by the following terms, and
 subject to these terms, Apple grants you a personal, non-exclusive
 license, under Apple's copyrights in this original Apple software (the
 "Apple Software"), to use, reproduce, modify and redistribute the Apple
 Software, with or without modifications, in source and/or binary forms;
 provided that if you redistribute the Apple Software in its entirety and
 without modifications, you must retain this notice and the following
 text and disclaimers in all such redistributions of the Apple Software.
 Neither the name, trademarks, service marks or logos of Apple Inc. may
 be used to endorse or promote products derived from the Apple Software
 without specific prior written permission from Apple.  Except as
 expressly stated in this notice, no other rights or licenses, express or
 implied, are granted by Apple herein, including but not limited to any
 patent rights that may be infringed by your derivative works or by other
 works in which the Apple Software may be incorporated.
 
 The Apple Software is provided by Apple on an "AS IS" basis.  APPLE
 MAKES NO WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION
 THE IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS
 FOR A PARTICULAR PURPOSE, REGARDING THE APPLE SOFTWARE OR ITS USE AND
 OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
 
 IN NO EVENT SHALL APPLE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL
 OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION,
 MODIFICATION AND/OR DISTRIBUTION OF THE APPLE SOFTWARE, HOWEVER CAUSED
 AND WHETHER UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE),
 STRICT LIABILITY OR OTHERWISE, EVEN IF APPLE HAS BEEN ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 
 Copyright (C) 2013 Apple Inc. All Rights Reserved.
 
 */

#import "JFAppDelegate.h"
#import "JFWrappedDocument.h"

NSString *UbiquityDirectoryComponentForDocuments = @"Documents";   //1

NSString *TextFileName = @"Text.txt";                        //2
NSString *LocationFileName = @"Location.txt";

NSStringEncoding kTextFileEncoding = NSUTF8StringEncoding;   //3

@interface JFWrappedDocument ()                              //4

@property (strong, nonatomic) NSFileWrapper *wrapperText;
@property (strong, nonatomic) NSFileWrapper *wrapperLocation;
@property (strong, nonatomic) NSFileWrapper *wrapperImage;

@property (strong, nonatomic) NSString *fwText;
@property (strong, nonatomic) NSString *fwLocation;

@end


@implementation JFWrappedDocument

+ (BOOL)autosavesInPlace
{
    // this gives us autosave and versioning for free in 10.7 and later
    return YES;
}

- (id)init
{
    self = [super init];
    if (self)
    {
    }
    return self;
}

-(void)setDocumentText: (NSString*)newText {     //5

  if (newText != _fwText )
  {
    NSString *oldText = _fwText;
    _fwText = newText;

    //register undo and cause autosave
    [self.undoManager setActionName: @"Text Change"];
    [self.undoManager registerUndoWithTarget:self
                                    selector: @selector(setDocumentText:)
                                      object: oldText];
  }
}

-(void)setDocumentLocation: (NSString*)newLocation {  //6
   if (newLocation != _fwLocation)
  {
    NSString *oldLocation = _fwLocation;
    _fwLocation = newLocation;
    
    [self.undoManager setActionName: @"Location Change"];
    [self.undoManager registerUndoWithTarget:self
                                    selector: @selector(setDocumentLocation:)
                                      object: oldLocation];
    
  }

}

-(NSString*)documentText {  //7
  if (_fwText)
    return _fwText;

  NSDictionary *fileWrappers = [_documentFileWrapper fileWrappers];   //8

  if (!_wrapperText) {
    _wrapperText = [fileWrappers objectForKey: TextFileName];  //9
  }
    
  if (_wrapperText != nil)
  {
    NSData *textData = [_wrapperText regularFileContents];  //10
    _fwText = [[NSString alloc] initWithData:textData encoding:kTextFileEncoding];  //11
  } else {
    // handle error
  }
  
  NSLog (@" %s %@", __PRETTY_FUNCTION__, _fwText);
  
  return _fwText;
}

-(NSString*)documentLocation {  //12
  if (_fwLocation)
    return _fwLocation;
  
  NSDictionary *fileWrappers = [_documentFileWrapper fileWrappers];
  
  if (!_wrapperLocation) {
    _wrapperLocation = [fileWrappers objectForKey: LocationFileName];
  }
  
  if (_wrapperLocation != nil)
  {
    NSData *locationData = [_wrapperLocation regularFileContents];
    _fwLocation = [[NSString alloc] initWithData:locationData encoding:kTextFileEncoding];
  } else {
    // handle error
  }
  
  NSLog (@" %s %@", __PRETTY_FUNCTION__, _fwLocation);

  return _fwLocation;
}

#pragma mark - File Wrapper/ Package Support

- (BOOL)loadFromContents:(id)contents ofType:(NSString *)typeName
                   error:(NSError *__autoreleasing *)outError {  //13
  
  self.documentFileWrapper = (NSFileWrapper *)contents;   //14
    
  if ([_delegate respondsToSelector: @selector (documentContentsDidChange:)]) { 
    [_delegate documentContentsDidChange: self];    //15
  }
    
  return YES;
}

- (id)contentsForType:(NSString *)typeName error:(NSError *__autoreleasing *)outError {
//  - (id)fileWrapperOfType:(NSString *)typeName error:(NSError *__autoreleasing *)outError {
  
  if (_documentFileWrapper == nil)             //16
  {
      _documentFileWrapper = [[NSFileWrapper alloc] initDirectoryWithFileWrappers:nil];  //17
  }

  NSDictionary *fileWrappersDictionary = [_documentFileWrapper fileWrappers];    //18
  NSLog (@" %s %@ %@ ", __PRETTY_FUNCTION__, _documentFileWrapper, fileWrappersDictionary);
  if (self.documentText != nil)  //19
  {
    NSData *textData = [self.documentText dataUsingEncoding: kTextFileEncoding];
    NSLog (@" %@",  self.documentText);
    NSFileWrapper *textFileWrapper = [fileWrappersDictionary objectForKey:TextFileName];
    if (textFileWrapper != nil) {
      //[_documentFileWrapper removeFileWrapper: textFileWrapper];
      [_documentFileWrapper removeFileWrapper: textFileWrapper];
    }
    NSLog (@" %@",  _documentFileWrapper);

    textFileWrapper = [[NSFileWrapper alloc] initRegularFileWithContents:textData];
    [textFileWrapper setPreferredFilename: TextFileName];
    //[self.documentFileWrapper addFileWrapper: textFileWrapper];
    [_documentFileWrapper addFileWrapper: textFileWrapper];
    NSLog (@" %@", _documentFileWrapper);
}
  
  if ( self.documentLocation != nil)  //20
  {
    NSData *locationData = [self.documentLocation dataUsingEncoding: kTextFileEncoding];
    NSLog (@" %@",  self.documentLocation);

    NSFileWrapper *locationFileWrapper = [fileWrappersDictionary objectForKey:LocationFileName];
    if (locationFileWrapper != nil) {
      [self.documentFileWrapper removeFileWrapper: locationFileWrapper];
    }
    locationFileWrapper = [[NSFileWrapper alloc] initRegularFileWithContents:locationData];
    NSLog (@" %@", locationData);
    [locationFileWrapper setPreferredFilename: LocationFileName];
    [self.documentFileWrapper addFileWrapper: locationFileWrapper];
  }
  NSLog (@" %@", _documentFileWrapper);

  return _documentFileWrapper;
}




@end
