//
//  JFDetailViewController.m
//  Placid
//
//  Created by Jesse Feiler on 7/3/13.
//  Copyright (c) 2013 Champlain Arts. All rights reserved.
//

#import "JFDetailViewController.h"
#import "JFFileRepresentation.h" 

@interface JFDetailViewController ()     //1

@property (strong, nonatomic) UIPopoverController *masterPopoverController;

@end

@implementation JFDetailViewController {  //2
  JFWrappedDocument *_document;
  BOOL _createFile;
}

#pragma mark - Managing the detail item

- (void)setDetailItemAndConfigure: (JFFileRepresentation *)fr {  //3
  [self unconfigureView];
  
  self.detailItem = fr;
  [self configureView];
}

- (void)configureView                 //4
{
  // Update the user interface for the detail item.
  if (self.detailItem) {
    NSURL *URLToSave = [self.detailItem fileURL];
    
    _document = [[JFWrappedDocument alloc ]initWithFileURL: URLToSave]; //5
    _document.delegate = self;
    
    if (_createFile) {          //6
      [_document saveToURL: _document.fileURL forSaveOperation: UIDocumentSaveForCreating completionHandler:^(BOOL success) {
        if (success) {     //7
          NSLog (@" %s %hhd", __PRETTY_FUNCTION__, success);
        }
      }];
      _createFile = NO;
    }
    
    if (_document.documentState & UIDocumentStateClosed) {   //8
      [_document openWithCompletionHandler:nil];
    }
  }
}

- (void)unconfigureView {    //9
  NSString *newText = self.textField.text;   //10
  NSString *newLocation = self.locationField.text;
  
  _document.documentText = newText;   //11
  _document.documentLocation = newLocation;

  [_document closeWithCompletionHandler:nil];  //12
}

- (void)viewDidLoad
{
    [super viewDidLoad];
	
    // Do any additional setup after loading the view, typically from a nib.
    [self configureView];     //13
}

- (void)viewWillAppear:(BOOL)animated {

  [super viewWillAppear:animated];
  
  [self configureView];      //14
  
}

- (void)viewWillDisappear:(BOOL)animated {
  
  [super viewWillDisappear:animated];
  
  [self unconfigureView];    //15
  
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - JFWrappedDocumentDelegate

- (void)documentContentsDidChange: (JFWrappedDocument *)document {  //16
  dispatch_async (dispatch_get_main_queue(), ^{    //17
    self.textField.text = _document.documentText;   //18
    self.locationField.text = _document.documentLocation;
  });
}

#pragma mark - Split view

- (void)splitViewController:(UISplitViewController *)splitController willHideViewController:(UIViewController *)viewController withBarButtonItem:(UIBarButtonItem *)barButtonItem forPopoverController:(UIPopoverController *)popoverController
{
    barButtonItem.title = NSLocalizedString(@"Master", @"Master");  //19
    [self.navigationItem setLeftBarButtonItem:barButtonItem animated:YES];
    self.masterPopoverController = popoverController;
}

- (void)splitViewController:(UISplitViewController *)splitController willShowViewController:(UIViewController *)viewController invalidatingBarButtonItem:(UIBarButtonItem *)barButtonItem
{
    // Called when the view is shown again in the split view, invalidating the button and popover controller.
    [self.navigationItem setLeftBarButtonItem:nil animated:YES];
    self.masterPopoverController = nil;
}

@end
