//
//  JFDocument.m
//  Chazy
//
//  Created by Champlain Arts Corp on 8/4/13.
//  Copyright (c) 2013 Champlain Arts Corp. All rights reserved.
//

#import "JFWrappedDocument.h"
#import "JFWindowController.h"

@interface JFWrappedDocument ()

@property (strong, nonatomic) NSFileWrapper *wrapperDescription;
@property (strong, nonatomic) NSFileWrapper *wrapperLocation;

@property (strong, nonatomic) NSString *fwDescription;
@property (strong, nonatomic) NSString *fwLocation;

@end

@implementation JFWrappedDocument

NSString *UbiquityDirectoryComponentForDocuments = @"Documents";   //1

NSString *descriptionFileName = @"Description.txt";   //2
NSString *locationFileName = @"Location.txt";

NSStringEncoding kTextFileEncoding = NSUTF8StringEncoding;   //3

- (id)init
{
  self = [super init];
  if (self) {
    
  // Add your subclass-specific initialization here.
  }
  
    return self;
}

/*
 - (NSString *)windowNibName *****
{
  // Override returning the nib file name of the document
  // If you need to use a subclass of NSWindowController or if your document supports multiple NSWindowControllers, you should remove this method and override -makeWindowControllers instead.
  return @"JFDocument";
}
 */

- (void)windowControllerDidLoadNib:(NSWindowController *)aController
{
  [super windowControllerDidLoadNib:aController];
  // Add any code here that needs to be executed once the windowController has loaded the document's window.
}

+ (BOOL)autosavesInPlace
{
    return YES;
}

- (void)controlTextDidChange:(NSNotification *)notification {
  NSLog (@" %s %@", __PRETTY_FUNCTION__, [notification name]);
  
  NSTextField *textField = [notification object];
  NSString *theText = [textField stringValue];
  self.documentDescription = theText;
  
  NSArray *myControllers = self.windowControllers;
  if ([myControllers count] > 0 ) {
    JFWindowController *firstController = myControllers [0];
    [firstController documentContentsDidChange:self ];
  }
}

#pragma mark - read/write

- (NSData *)dataOfType:(NSString *)typeName error:(NSError **)outError
{
  // Insert code here to write your document to data of the specified type. If outError != NULL, ensure that you create and set an appropriate error when returning nil.
  // You can also choose to override -fileWrapperOfType:error:, -writeToURL:ofType:error:, or -writeToURL:ofType:forSaveOperation:originalContentsURL:error: instead.
  NSException *exception = [NSException exceptionWithName:@"UnimplementedMethod" reason:[NSString stringWithFormat:@"%@ is unimplemented", NSStringFromSelector(_cmd)] userInfo:nil];
  @throw exception;
  return nil;
}

//debug
- (void)makeWindowControllers {
  //[super makeWindowControllers];
  
  JFWindowController *myController;
  
  myController = [[JFWindowController alloc] init]; //*****
  [self addWindowController: myController];
  [myController setDocument: self];
  self.windowController = myController;
  NSLog (@" %s %@", __PRETTY_FUNCTION__, self.windowControllers[0]);
  

}

- (void)showWindows {
  [super showWindows];
  
  NSLog (@" %s, %@ %@", __PRETTY_FUNCTION__, self.documentDescription, self.documentLocation);
 
  [self.windowController documentContentsDidChange:self];

}

/*
 - (BOOL)readFromData:(NSData *)data ofType:(NSString *)typeName error:(NSError **)outError
{
  // Insert code here to read your document from the given data of the specified type. If outError != NULL, ensure that you create and set an appropriate error when returning NO.
  // You can also choose to override -readFromFileWrapper:ofType:error: or -readFromURL:ofType:error: instead.
  // If you override either of these, you should also override -isEntireFileLoaded to return NO if the contents are lazily loaded.
  NSException *exception = [NSException exceptionWithName:@"UnimplementedMethod" reason:[NSString stringWithFormat:@"%@ is unimplemented", NSStringFromSelector(_cmd)] userInfo:nil];
  @throw exception;
  return YES;
}
*/

- (BOOL)readFromFileWrapper: (NSFileWrapper *)fileWrapper
                     ofType: (NSString *)typeName
                      error: (NSError **)outError
{  //loadFromContents
  
  _fwDescription = nil;
  _fwLocation = nil;
  _documentFileWrapper = fileWrapper;  //15
  
  if ([_delegate respondsToSelector: @selector (documentContentsDidChange:)]) {
    [_delegate documentContentsDidChange: self];    //16
  }
  
  return YES;
}

- (NSFileWrapper *)fileWrapperOfType: (NSString *)typeName
                               error:(NSError **)outError {
  
  if (_documentFileWrapper == nil)             //17
  {
    _documentFileWrapper = [[NSFileWrapper alloc] initDirectoryWithFileWrappers:nil];  //18
  }
  
  NSDictionary *fileWrappersDictionary = [_documentFileWrapper fileWrappers];    //19
  
  if (self.documentDescription != nil)  //20
    
  {
    
    NSLog (@" %s textData %@", __PRETTY_FUNCTION__, self.documentDescription);
    
    NSData *textData = [self.documentDescription dataUsingEncoding: kTextFileEncoding];
    
    NSFileWrapper *textFileWrapper = [fileWrappersDictionary objectForKey:descriptionFileName];
    if (textFileWrapper != nil) {
      [_documentFileWrapper removeFileWrapper: textFileWrapper];
    }
    textFileWrapper = [[NSFileWrapper alloc] initRegularFileWithContents:textData];
    [textFileWrapper setPreferredFilename: descriptionFileName];
    [_documentFileWrapper addFileWrapper: textFileWrapper];
  }
  
  if ( self.documentLocation != nil)  //21
  {
    NSLog (@" %s locationData %@", __PRETTY_FUNCTION__, self.documentLocation);
    
    NSData *locationData = [self.documentLocation dataUsingEncoding: kTextFileEncoding];
    
    NSFileWrapper *locationFileWrapper = [fileWrappersDictionary objectForKey:locationFileName];
    if (locationFileWrapper != nil) {
      [_documentFileWrapper removeFileWrapper: locationFileWrapper];
    }
    locationFileWrapper = [[NSFileWrapper alloc] initRegularFileWithContents:locationData];
    [locationFileWrapper setPreferredFilename: locationFileName];
    [_documentFileWrapper addFileWrapper: locationFileWrapper];
  }
  
  return _documentFileWrapper;
  
}

#pragma mark - accessors

-(void)setDocumentDescription: (NSString*)newDescription {     //5
  
  if (newDescription != _fwDescription )
  {
    NSString *oldDescripion = _fwDescription;
    _fwDescription = newDescription;
    
    //register undo and cause autosave
    [self.undoManager setActionName: @"Description Change"];
    [self.undoManager registerUndoWithTarget:self
                                    selector: @selector(setDocumentDescription:)
                                      object: oldDescripion];
  }
}

-(void)setDocumentLocation: (NSString*)newLocation {  //6
  if (newLocation != _fwLocation)
  {
    NSString *oldLocation = _fwLocation;
    _fwLocation = newLocation;
    
    [self.undoManager setActionName: @"Location Change"];
    [self.undoManager registerUndoWithTarget:self
                                    selector: @selector(setDocumentLocation:)
                                      object: oldLocation];
    
  }
  
}

-(NSString*)documentDescription {  //7
  if (_fwDescription)
    return _fwDescription;
  
  NSDictionary *fileWrappers = [_documentFileWrapper fileWrappers];   //8
  
  if (!_wrapperDescription) {
    _wrapperDescription = [fileWrappers objectForKey: descriptionFileName];  //9
  }
  
  if (_wrapperDescription != nil)
  {
    NSData *textData = [_wrapperDescription regularFileContents];  //10
    _fwDescription = [[NSString alloc] initWithData:textData encoding:kTextFileEncoding];  //11
  } else {
    // handle error
  }
  
  return _fwDescription;
}

-(NSString*)documentLocation {  //12
  if (_fwLocation)
    return _fwLocation;
  
  NSDictionary *fileWrappers = [_documentFileWrapper fileWrappers];   //8
  
  if (!_wrapperDescription) {
    _wrapperDescription = [fileWrappers objectForKey: locationFileName];  //9
  }
  
  if (_wrapperDescription != nil)
  {
    NSData *locationData = [_wrapperDescription regularFileContents];  //10
    _fwLocation = [[NSString alloc] initWithData:locationData encoding:kTextFileEncoding];  //11
  } else {
    // handle error
  }
  
  return _fwLocation;
}

@end
