//
//  JFWrappedDocument.m
//  Chazy
//
//  Created by Champlain Arts Corp on 8/4/13.
//  Copyright (c) 2013 Champlain Arts Corp. All rights reserved.
//

#import "JFWrappedDocument.h"
#import "JFWindowController.h"

@interface JFWrappedDocument ()

  @property (strong, nonatomic) NSFileWrapper *documentFileWrapper;

  @property (strong, nonatomic) NSFileWrapper *wrapperDescription;
  @property (strong, nonatomic) NSFileWrapper *wrapperLocation;

  @property (strong, nonatomic) NSString *fwDescription;
  @property (strong, nonatomic) NSString *fwLocation;

@end

@implementation JFWrappedDocument

NSString *UbiquityDirectoryComponentForDocuments = @"Documents";   //1

NSString *descriptionFileName = @"Description.txt"; //2
NSString *locationFileName = @"Location.txt";

NSStringEncoding kTextFileEncoding = NSUTF8StringEncoding;   //3

#pragma mark - initialization

- (id)init
{
  self = [super init];
  if (self) {
    
  // Add your subclass-specific initialization here.
  }
  
    return self;
}

/*
 - (NSString *)windowNibName //4
{
  // Override returning the nib file name of the document
  // If you need to use a subclass of NSWindowController or if your document supports multiple NSWindowControllers, you should remove this method and override -makeWindowControllers instead.
    
    return @"JFDocument";
}
 */

- (void)windowControllerDidLoadNib:(NSWindowController *)aController
{
  [super windowControllerDidLoadNib:aController];
  // Add any code here that needs to be executed once the windowController has loaded the document's window.
}

+ (BOOL)autosavesInPlace
{
    return YES;               //5
}

#pragma mark - windows
- (void)makeWindowControllers {         //6
  
  JFWindowController *myController;
  
  myController = [[JFWindowController alloc] init];
  [self addWindowController: myController];
  [myController setDocument: self];
  self.windowController = myController;
  
}

- (void)showWindows {               //7
  [super showWindows];
  
  [self.windowController documentContentsDidChange:self];
  
}

#pragma mark - read/write

- (BOOL)readFromFileWrapper: (NSFileWrapper *)fileWrapper  //8
                     ofType: (NSString *)typeName
                      error: (NSError **)outError
{
  
  self.documentFileWrapper = fileWrapper;   //9 cf contents

  return YES; //self.fileWrapper;
}

- (NSFileWrapper *)fileWrapperOfType: (NSString *)typeName error:(NSError **)outError {
  if (self.documentFileWrapper == nil)             //10
  {
    self.documentFileWrapper = [[NSFileWrapper alloc] initDirectoryWithFileWrappers:nil];  //11
  }
  
  NSDictionary *fileWrappers = [self.documentFileWrapper fileWrappers];    //12
  
  if (self.documentDescription != nil)  //13
  {
    NSData *descriptionData = [self.documentDescription dataUsingEncoding: kTextFileEncoding];
    
    NSFileWrapper *descriptionFileWrapper = [fileWrappers objectForKey:descriptionFileName];
    if (descriptionFileWrapper != nil) {
      [self.documentFileWrapper removeFileWrapper: descriptionFileWrapper];
    }
    descriptionFileWrapper = [[NSFileWrapper alloc] initRegularFileWithContents:descriptionData];
    [descriptionFileWrapper setPreferredFilename: descriptionFileName];
    [self.documentFileWrapper addFileWrapper: descriptionFileWrapper];
  }
  
  if ( self.documentLocation != nil)  //14
  {
    NSData *locationData = [self.documentLocation dataUsingEncoding: kTextFileEncoding];
    
    NSFileWrapper *locationFileWrapper = [fileWrappers objectForKey:locationFileName];
    if (locationFileWrapper != nil) {
      [self.documentFileWrapper removeFileWrapper: locationFileWrapper];
    }
    locationFileWrapper = [[NSFileWrapper alloc] initRegularFileWithContents:locationData];
    [locationFileWrapper setPreferredFilename: locationFileName];
    [self.documentFileWrapper addFileWrapper: locationFileWrapper];
  }
  
  return self.documentFileWrapper; //15
}

#pragma mark - accessors

-(void)setDocumentDescription: (NSString*)newText {     //16
  
  if (newText != _fwDescription )
  {
    NSString *oldText = _fwDescription;
    _fwDescription = newText;
    
    //register undo and cause autosave
    [self.undoManager setActionName: @"Text Change"];
    [self.undoManager registerUndoWithTarget:self
                                    selector: @selector(setDocumentDescription:)
                                      object: oldText];
  }
}

-(void)setDocumentLocation: (NSString*)newLocation {  //17
  if (newLocation != _fwLocation)
  {
    NSString *oldLocation = _fwLocation;
    _fwLocation = newLocation;
    
    [self.undoManager setActionName: @"Location Change"];
    [self.undoManager registerUndoWithTarget:self
                                    selector: @selector(setDocumentLocation:)
                                      object: oldLocation];
    
  }
  
}

-(NSString*)documentDescription {  //18
  if (_fwDescription)
    return _fwDescription;
  
  NSDictionary *fileWrappers = [_documentFileWrapper fileWrappers];   //19
  
  if (!_wrapperDescription) {
    _wrapperDescription = [fileWrappers objectForKey: descriptionFileName];  //20
  }
  
  if (_wrapperDescription != nil)
  {
    NSData *descriptionData = [_wrapperDescription regularFileContents];  //21
    _fwDescription = [[NSString alloc] initWithData:descriptionData encoding:kTextFileEncoding];  //22
  } else {
    // handle error
  }
  
  return _fwDescription;
}

-(NSString*)documentLocation {  //23
  if (_fwLocation)
    return _fwLocation;
  
  NSDictionary *fileWrappers = [_documentFileWrapper fileWrappers];
  
  if (!_wrapperLocation) {
    _wrapperLocation = [fileWrappers objectForKey: locationFileName];
  }
  
  if (_wrapperLocation != nil)
  {
    NSData *locationData = [_wrapperLocation regularFileContents];
    _fwLocation = [[NSString alloc] initWithData:locationData encoding:kTextFileEncoding]; 
  } else {
    // handle error
  }
  
  return _fwLocation;
}

@end
