//
//  JFAppDelegate.m
//  Placid
//
//  Created by Jesse Feiler on 7/3/13.
//  Copyright (c) 2013 Champlain Arts. All rights reserved.
//

#import "Constants.h"
#import "JFAppDelegate.h"

@interface JFAppDelegate ()

@property id ubiquityToken;                   //1
@property NSURL *ubiquityContainer;
@property (readwrite) BOOL documentsIniCloud; //2
@property (nonatomic, readwrite) SharediCloudController *iCloudController;  //*******

@end

@implementation JFAppDelegate

#pragma mark = SharediCloudController

- (SharediCloudController *)iCloudController {
  if (!_iCloudController) {                                       //2
    _iCloudController = [[SharediCloudController alloc] init];
  }
  return _iCloudController;
}

#pragma mark - launch and background

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
  
  if (![[NSUserDefaults standardUserDefaults] boolForKey: @"com.champlainarts.Loon.hasLaunched"]) {  //3
    [[NSUserDefaults standardUserDefaults] setBool:YES forKey: @"com.champlainarts.Loon.hasLaunched"];
    [[NSUserDefaults standardUserDefaults] synchronize];
    NSString *msg = @"Should documents be stored in iCloud and available on all your devices?"; // 4 for spacing
    
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle: @"Choose Storage Option"            //5
                                                    message: msg
                                                   delegate: self
                                          cancelButtonTitle: @"Local Only"
                                          otherButtonTitles: @"Use iCloud", nil];
    [alert show];
    
    }
  
  dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
  _ubiquityContainer = [[NSFileManager defaultManager]URLForUbiquityContainerIdentifier:nil]; //6 check for
  //icloud avail and store token
  if (_ubiquityContainer != nil) {
    NSLog (@"Got iCloud\n");
    _documentsIniCloud = YES;
    dispatch_async (dispatch_get_main_queue(), ^(void) {                  //7 main queue for UI
    });
  }
  else
  {
    NSLog (@"No iCloud\n");
    _documentsIniCloud = NO;
  }
  });
  
  _documentsIniCloud = [[NSUserDefaults standardUserDefaults] boolForKey: @"com.champlainarts.Loon.documentsIniCloud"]; //8
  
  NSLog (@" %s, %hhd", __PRETTY_FUNCTION__, _documentsIniCloud);       //9
  
  if (_documentsIniCloud){
    [self initializeiCloudAccess];                                       //10
  };
  
  if (!_documentsIniCloud) { // 11 
    
  }
  
  // Override point for customization after application launch.
  if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPad) {
    UISplitViewController *splitViewController = (UISplitViewController *)self.window.rootViewController;
    UINavigationController *navigationController = [splitViewController.viewControllers lastObject];
    splitViewController.delegate = (id)navigationController.topViewController;
  }
  
  return YES;
}
							
- (void)applicationWillResignActive:(UIApplication *)application
{
  // Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
  // Use this method to pause ongoing tasks, disable timers, and throttle down OpenGL ES frame rates. Games should use this method to pause the game.
}

- (void)applicationDidEnterBackground:(UIApplication *)application
{
  // Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later.
  // If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
}

- (void)applicationWillEnterForeground:(UIApplication *)application
{
  // Called as part of the transition from the background to the inactive state; here you can undo many of the changes made on entering the background.
}

- (void)applicationDidBecomeActive:(UIApplication *)application
{
  // Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
}

- (void)applicationWillTerminate:(UIApplication *)application
{
  // Called when the application is about to terminate. Save data if appropriate. See also applicationDidEnterBackground:.
}

#pragma mark - iCloud

- (void)initializeiCloudAccess {
  _ubiquityToken = [[NSFileManager defaultManager]ubiquityIdentityToken];    //12
  
  if (_ubiquityToken) {                         //13
    NSData *newTokenData = [NSKeyedArchiver archivedDataWithRootObject: _ubiquityToken]; // archive
    [[NSUserDefaults standardUserDefaults] //defaults
     setObject:newTokenData
     forKey: @"com.champlainarts.Loon.UbiquityIdentityToken"];
    [[NSUserDefaults standardUserDefaults] synchronize];
  } else {
    [[NSUserDefaults standardUserDefaults]      //  14 remove old token -- for airplane mode, get same token but
     // no iCloud sign out or turn off Documents & Data means no token.
     removeObjectForKey: @"com.champlainarts.Loon.UbiquityIdentityToken"];
    [[NSUserDefaults standardUserDefaults] synchronize];
    
  }
  
  [[NSNotificationCenter defaultCenter] addObserver:self         //15
                                           selector:@selector (ubiquityIdentityChanged)
                                               name: NSUbiquityIdentityDidChangeNotification
                                             object:nil];
}

#pragma mark - NSUbiquityIdentityDidChangeNotification

- (void) ubiquityIdentityChanged { // 16
  
}

#pragma mark - UIAlertViewDelegate

- (void)alertView:(UIAlertView *)alertView didDismissWithButtonIndex:(NSInteger)buttonIndex {  //17
  if (buttonIndex == [alertView firstOtherButtonIndex]) { // other button
    _documentsIniCloud = YES;
  } else {
    _documentsIniCloud = NO;
  }
  
  NSLog (@" %hhd", _documentsIniCloud);
  
  
  [[NSUserDefaults standardUserDefaults] //defaults
   setBool:_documentsIniCloud
   forKey: @"com.champlainarts.Loon.documentsIniCloud"];

}

@end
