//
//  JFMasterViewController.m
//  Placid
//
//  Created by Jesse Feiler on 7/3/13.
//  Copyright (c) 2013 Champlain Arts. All rights reserved.
//

#import "JFMasterViewController.h"
#import "JFDetailViewController.h"
#import "JFWrappedDocument.h"
#import "JFAppDelegate.h"
#import "JFFileRepresentation.h"
#import "JFAppDelegate.h"
#import "SharediCloudController.h"
#import "Constants.h"

static NSString *JFDocumentsDirectoryName = @"Documents";       //3
static NSString *DocumentEntryCell = @"DocumentEntryCell";      //4

@interface JFMasterViewController () {
  NSMetadataQuery *_query;                                         //5
}

@property (strong, nonatomic) SharediCloudController *controller;         //6

@end

@implementation JFMasterViewController

#pragma mark - Load and memory

- (void)awakeFromNib
{
  
  [super awakeFromNib];
  
  if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPad) {
    self.clearsSelectionOnViewWillAppear = NO;
    self.preferredContentSize = CGSizeMake(320.0, 600.0);
  }
  
}

- (void)viewDidLoad
{
  [super viewDidLoad];
	// Do any additional setup after loading the view, typically from a nib.

  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector (reloadData) name:SharediCloudControllerDidChangeDataNotification object:nil];           //10

  self.navigationItem.leftBarButtonItem = self.editButtonItem;
  self.navigationItem.leftBarButtonItem = self.editButtonItem;             //7
  UIBarButtonItem* addButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemAdd target:self action:@selector(addDocument:)];                          //8
  self.navigationItem.rightBarButtonItem = addButton;                    //9
  self.addButton = addButton;              //10
  self.detailViewController = (JFDetailViewController *)[[self.splitViewController.viewControllers lastObject] topViewController];
  
  BOOL usingiCloud = ((JFAppDelegate *)[[UIApplication sharedApplication] delegate]).documentsIniCloud;
  _controller = ((JFAppDelegate *)[[UIApplication sharedApplication] delegate]).iCloudController;

  if (!_controller.documents)
    _controller.documents = [[NSMutableArray alloc] init];                             //11
  
  if (usingiCloud) {
    [_controller setupAndStartQuery];
  } else {
    [self fillDocumentsForLocalStorage]; //13
  }

}

- (void)reloadData {
  [self.tableView reloadData];
  NSLog (@"From Data Changed %@", _controller.documents);
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}


/*
 - (void)insertNewObject:(id)sender
{
    if (!_objects) {
        _objects = [[NSMutableArray alloc] init];
    }
    [_objects insertObject:[NSDate date] atIndex:0];
    NSIndexPath *indexPath = [NSIndexPath indexPathForRow:0 inSection:0];
    [self.tableView insertRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationAutomatic];
}
*/

#pragma mark - Documents

- (IBAction)addDocument:(id)sender {                                                 //12
  NSURL *newDocumentURL;
  
  BOOL usingiCloud = ((JFAppDelegate *)[[UIApplication sharedApplication] delegate]).documentsIniCloud;
  if (usingiCloud) { // for clarity
    self.addButton.enabled = NO;
    
    dispatch_async (dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{  //13
      
      NSFileManager *fm = [NSFileManager defaultManager];
      NSString *newFileName = [self newUntitledDocumentName];
      NSURL *newDocumentURL = [fm URLForUbiquityContainerIdentifier:nil];              //14
      NSLog (@" %s %@", __PRETTY_FUNCTION__, [fm URLForUbiquityContainerIdentifier:nil]);
      newDocumentURL = [newDocumentURL URLByAppendingPathComponent:JFDocumentsDirectoryName isDirectory:YES];  //15
      newDocumentURL = [newDocumentURL URLByAppendingPathComponent:newFileName];  //16
      NSLog (@" %s %@", __PRETTY_FUNCTION__, newDocumentURL);

      JFFileRepresentation *fr = [[JFFileRepresentation alloc] initWithFileName: newFileName fileURL:newDocumentURL];

      [_controller.documents addObject: fr];
      
      NSLog (@" %s %@", __PRETTY_FUNCTION__, _controller.documents);
      
    
      dispatch_async(dispatch_get_main_queue(), ^{                                     //17
        
        //update table                                               //18
         NSIndexPath* newCellIndexPath =
        [NSIndexPath indexPathForRow: ([_controller.documents count] - 1) inSection: 0];
        [self.tableView insertRowsAtIndexPaths: [NSArray arrayWithObject: newCellIndexPath]
                              withRowAnimation:UITableViewRowAnimationAutomatic];
        
        [self.tableView selectRowAtIndexPath:newCellIndexPath animated:YES scrollPosition:(UITableViewScrollPositionMiddle)];
        UITableViewCell *selectedCell = [self.tableView cellForRowAtIndexPath: newCellIndexPath];
        if ([[UIDevice currentDevice] userInterfaceIdiom] != UIUserInterfaceIdiomPad) { //19
          [self performSegueWithIdentifier:DisplayDetailSegue sender:selectedCell];
           } else {
            [self.detailViewController prepareToCreateFile];
            [self.detailViewController setDetailItemAndConfigure:fr];
            _detailViewController.navigationItem.title = [[fr fileName]stringByDeletingPathExtension];
        } 
        
        self.addButton.enabled = YES;                                                  //20
      });
    });
  } else {
    newDocumentURL = [self localDocumentsDirectoryURL];         //21
    NSLog (@" %s %@", __PRETTY_FUNCTION__, newDocumentURL);
    newDocumentURL = [newDocumentURL URLByAppendingPathComponent: [self newUntitledDocumentName]];                                //22
    
    NSLog (@" %s %@", __PRETTY_FUNCTION__, newDocumentURL);
    JFFileRepresentation *fr = [[JFFileRepresentation alloc] initWithFileName: [self newUntitledDocumentName] fileURL:newDocumentURL];
    [_controller.documents addObject: fr];                                 //23
    
    NSLog (@" %s %@", __PRETTY_FUNCTION__, _controller.documents);         //24
    NSIndexPath* newCellIndexPath =
    [NSIndexPath indexPathForRow: ([_controller.documents count] - 1) inSection: 0];
    [self.tableView insertRowsAtIndexPaths: [NSArray arrayWithObject: newCellIndexPath]
    withRowAnimation:UITableViewRowAnimationAutomatic];

    [self.tableView selectRowAtIndexPath:newCellIndexPath animated:YES scrollPosition:(UITableViewScrollPositionMiddle)];        //25
    UITableViewCell *selectedCell = [self.tableView cellForRowAtIndexPath: newCellIndexPath];
    
    if ([[UIDevice currentDevice] userInterfaceIdiom] != UIUserInterfaceIdiomPad) {                                //26
      [self performSegueWithIdentifier:DisplayDetailSegue sender:selectedCell];
    } else {
      [_detailViewController setDetailItemAndConfigure: fr];    //27
      _detailViewController.navigationItem.title = [fr fileName];
    }
    
     self.addButton.enabled = YES;
  }
}

- (NSString *)newUntitledDocumentName {
  
  NSInteger docCount = 1;
  NSString *newDocName = nil;
  
  // Manage document list updates
  BOOL done = NO;
  
  while (!done) {
    newDocName = [NSString stringWithFormat: @"Note %d", docCount];                              //28
     
    //search for first unused docCount value
    BOOL nameExists = NO;
    for (JFFileRepresentation *fr in _controller.documents) {          //29
      NSLog (@" %@ %@", [[fr fileName] lastPathComponent], newDocName);
      if ([[[fr fileName] lastPathComponent] isEqualToString: newDocName]) {
        docCount++;
        nameExists = YES;
        break;
      }
    }
    
    if (!nameExists)
      done = YES;
  }
  
  newDocName = [NSString stringWithFormat: @"%@.%@", newDocName, SharedReportFileExtension];
  NSLog (@" %s %@", __PRETTY_FUNCTION__, newDocName);
  
  return newDocName;
  
}

#pragma mark - Table View

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
  return 1;                                                        //30
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return _controller.documents.count;                                        //31
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
  UITableViewCell *newCell = [tableView dequeueReusableCellWithIdentifier:DocumentEntryCell forIndexPath:indexPath];                                                     //32
  
  if (!newCell)
    newCell = [[UITableViewCell alloc] initWithStyle: UITableViewCellStyleDefault reuseIdentifier: DocumentEntryCell]; //33

  if (!newCell)
    return nil;
  
  JFFileRepresentation *fr = [_controller.documents objectAtIndex:[indexPath row]];
  NSLog (@" %@", fr);
  NSLog (@" %s %@ %@", __PRETTY_FUNCTION__, fr.fileURL, fr.fileName);

  newCell.textLabel.text = [[fr.fileName lastPathComponent] stringByDeletingPathExtension];                               //35
  NSLog (@" %s %@", __PRETTY_FUNCTION__, newCell.textLabel.text);
  return newCell;
}

- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the specified item to be editable.
    return YES;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
  JFFileRepresentation *fr = (JFFileRepresentation *)[_controller.documents objectAtIndex: [indexPath row]];                                   //36
  
  [self.detailViewController setDetailItemAndConfigure:fr];   //37
}


#pragma mark - Segue

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
  
  if (![segue.identifier isEqualToString:@"showDetail"])
      return;
  
  JFDetailViewController *destVC = (JFDetailViewController *)segue.destinationViewController;
  
  NSIndexPath *cellPath = [self.tableView indexPathForSelectedRow];
  UITableViewCell *theCell = [self.tableView cellForRowAtIndexPath: cellPath];
  JFFileRepresentation *fr = (JFFileRepresentation *)[_controller.documents objectAtIndex: [cellPath row]];                                   ///38
  
  destVC.detailItem = fr;                                                                  //39
  destVC.navigationItem.title = theCell.textLabel.text;                                        //40
  
}

#pragma mark - iCloud Query

- (void)setupAndStartQuery {                   //41
  if (!_query)
    _query = [self wrappedDocumentQuery];
  [[NSNotificationCenter defaultCenter] addObserver: self selector:@selector (processFiles) name: NSMetadataQueryDidFinishGatheringNotification object: nil];
  [[NSNotificationCenter defaultCenter] addObserver: self selector:@selector (processFiles) name: NSMetadataQueryDidUpdateNotification object: nil];
  
  [_query startQuery];
  
}

- (NSMetadataQuery *)wrappedDocumentQuery {   //42
  NSMetadataQuery *aQuery = [[NSMetadataQuery alloc] init];
  if (aQuery) {
    [aQuery setSearchScopes: [NSArray arrayWithObject: NSMetadataQueryUbiquitousDocumentsScope]]; //43
    NSString *filePattern = [NSString stringWithFormat: @"*.%@", SharedReportFileExtension];
    NSLog (@" %s %@", __PRETTY_FUNCTION__,
           filePattern);
    
    [aQuery setPredicate: [NSPredicate predicateWithFormat:@"%K LIKE %@", NSMetadataItemFSNameKey, filePattern]];
  }
  return aQuery;
}

- (void)processFiles {                      //44
  
  //preserve selection listing 4-4 doc
  
  NSMutableArray *discoveredFiles = [NSMutableArray array];   //45
  
  [_query disableUpdates];                         //46
  
  NSArray *queryResults = [_query results];
  for (NSMetadataItem *result in queryResults) {
    NSString *fileName = [result valueForAttribute: NSMetadataItemFSNameKey];
    NSURL *fileURL = [result valueForAttribute: NSMetadataItemURLKey];
    NSLog (@" %s %@ %@", __PRETTY_FUNCTION__, fileName, fileURL);
    NSNumber *aBool = nil;
    [fileURL getResourceValue: &aBool forKey: NSURLIsHiddenKey error: nil];
    if (aBool && ![aBool boolValue])       {  //47
      JFFileRepresentation *fr = [[JFFileRepresentation alloc] initWithFileName: fileName fileURL:fileURL];
    [discoveredFiles addObject: fr];
    }
  }
  
  [_controller.documents removeAllObjects];   //48
  [_controller.documents addObjectsFromArray: discoveredFiles];
  [self.tableView reloadData];
  
  [_query enableUpdates];     //49
  
}

#pragma mark - Local Sandbox

- (void)fillDocumentsForLocalStorage {
  NSMutableArray *discoveredFiles = [NSMutableArray array];   //50
  
  NSArray *localDocumentsArray = [[NSFileManager defaultManager] //51
                                  URLsForDirectory:NSDocumentDirectory inDomains:NSUserDomainMask];
  NSURL *documentsDirectoryURL = localDocumentsArray[0];
  NSError *error = nil;
  NSArray *properties = @[NSURLNameKey];
  NSArray *documents = [[NSFileManager defaultManager] contentsOfDirectoryAtURL:documentsDirectoryURL includingPropertiesForKeys:properties options:NSDirectoryEnumerationSkipsHiddenFiles error:&error];
    
  for( NSURL *myFileURL in documents) {               //52
    NSString *fileName = [myFileURL lastPathComponent];
    NSLog (@" %s %@ %@", __PRETTY_FUNCTION__, fileName, myFileURL);
    [discoveredFiles addObject: [[JFFileRepresentation alloc]
                                  initWithFileName: fileName
                                  fileURL: myFileURL
                                   ]];
  }
  
  [_controller.documents removeAllObjects];   //53
  [_controller.documents addObjectsFromArray: discoveredFiles];
  [self.tableView reloadData];

}

- (NSURL*)localDocumentsDirectoryURL {
   static NSURL *localDocumentsDirectoryURL = nil;
  
  if (localDocumentsDirectoryURL == nil) {
    NSString *documentsDirectoryPath = [NSSearchPathForDirectoriesInDomains(
                                                                            NSDocumentDirectory,
                                                                            NSUserDomainMask,
                                                                            YES ) objectAtIndex:0];
    localDocumentsDirectoryURL = [NSURL fileURLWithPath:documentsDirectoryPath];
    
     NSLog (@"%s %@", __PRETTY_FUNCTION__, localDocumentsDirectoryURL);
   
  }
  
  return localDocumentsDirectoryURL;
}


@end

