//
//  SharediCloudController.m
//  OS X iCloud Reports
//
//  Created by Jesse Feiler on 3/17/13.
//  Copyright (c) 2013 Champlain Arts Corp. All rights reserved.
//

#import "SharediCloudController.h"
#import "JFFileRepresentation.h"
#import "Constants.h"

@interface SharediCloudController ()            //1

@property NSMetadataQuery *query;

@end

@implementation SharediCloudController

#pragma mark - initialization

-(id)init {
  if ( self = [super init] ) {
    [self initializeiCloudAccess];                 //2
    
    if (!_documents)
      _documents = [[NSMutableArray alloc] init];  //3
    
    [self setupAndStartQuery];                     //4
  }
  return self;
}

- (void)initializeiCloudAccess {                   //5
	dispatch_async (dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0),
    ^{
        if ([[NSFileManager defaultManager] URLForUbiquityContainerIdentifier:nil] != nil)
          NSLog (@"Got iCloud with URLForUbiquityContainerIdentifier %@ \n",
                 [[NSFileManager defaultManager] URLForUbiquityContainerIdentifier:nil]);
        else
          NSLog (@"No iCloud");
      });
}

#pragma mark - managing iCloud data

-(NSMetadataQuery *)textDocumentQuery {
  NSMetadataQuery *aQuery = [[NSMetadataQuery alloc] init];
  if (aQuery) {
    // search
    [aQuery setSearchScopes: [NSArray arrayWithObject: NSMetadataQueryUbiquitousDocumentsScope]];                                           //6
    
    // add predicate
    NSString * filePattern = [NSString stringWithFormat:@"*.%@", SharedReportFileExtension]; //7
    [aQuery setPredicate: [NSPredicate predicateWithFormat: @"%K LIKE %@", NSMetadataItemFSNameKey, filePattern]];                                              //8
  }
  //NSLog (@" %@", aQuery);
  return aQuery;
}

- (void)setupAndStartQuery {
  // create if needed
  if (!_query)
    _query = [self textDocumentQuery];                                     //9
  
  // register for query notifications
  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector (processFiles:) name:NSMetadataQueryDidFinishGatheringNotification object:nil];           //10
  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector (processFiles:) name:NSMetadataQueryDidUpdateNotification object:nil];                    //11
  //start the query
  [_query startQuery];                                                     //12
  
}

-(void)processFiles: (NSNotification*)aNotification {
  NSMutableArray *discoveredFiles = [NSMutableArray array];
  
  //disable updates
  [_query disableUpdates];                                                 //13
  
  
  NSArray *queryResults = [_query results];                                //14
  
  for (NSMetadataItem *result in queryResults) {
    NSURL *fileURL = [result valueForAttribute: NSMetadataItemURLKey];
    NSNumber *aBool = nil;
    
    //don't include hidden files
    [fileURL getResourceValue: &aBool forKey:NSURLIsHiddenKey error:nil];  //15
    if (aBool && ![aBool boolValue])  {
      JFFileRepresentation *fr = [[JFFileRepresentation alloc] initWithFileName: [[fileURL lastPathComponent] stringByDeletingPathExtension] fileURL:fileURL];
      NSLog (@" %s %@ %@", __PRETTY_FUNCTION__, fr.fileName, fr.fileURL);
      //[discoveredFiles addObject: fileURL];
      [discoveredFiles addObject: fr];
    }
  }
  
  [_documents removeAllObjects];                                           //16
  [_documents addObjectsFromArray: discoveredFiles];
  
  [_query enableUpdates];                                                  //17
  
  [[NSNotificationCenter defaultCenter]
     postNotificationName: SharediCloudControllerDidChangeDataNotification
     object:nil userInfo:nil];                                             //18
}

#pragma mark - managing documents array

-(NSString*)stringForDocumentNameAt: (NSInteger)index {                    //20
  if ([_documents count] > 0)
    {
    NSURL *theURL = [_documents objectAtIndex: index];
    return [[theURL lastPathComponent] stringByDeletingPathExtension];
    }
  return nil;
}

-(NSURL *)documentURLAt: (NSInteger)index {                                //21
  if ([_documents count] > 0)
    {
    NSURL *theURL = [_documents objectAtIndex: index];
    return theURL;
    }
  return nil;
}

- (NSInteger)numberOfDocuments {                                           //22
  return [_documents count];
}

- (NSString *)newUntitledDocumentName {                                    //23
	NSInteger docCount = 1;
	NSString *newDocName = nil;
	  
  newDocName = @"new doc";
  BOOL done = NO;
  while (!done) {
    newDocName = [NSString stringWithFormat: @"Note %d.%@", docCount, SharedReportFileExtension];
    // check for dups
    BOOL nameExists = NO;
    for (NSURL *aURL in _documents) {
      NSLog(@" %@ %@", [aURL lastPathComponent], newDocName);
      if ([[aURL lastPathComponent] isEqualToString: newDocName]) {
        docCount ++;
        nameExists = YES;
        break;
      }
    }
 
    // if not found, exit WHILE
    if (!nameExists)
      done = YES;
  }
  return newDocName;
}

- (void)addDocument: (NSURL *)newDocumentURL {                          //24
  [_documents addObject: newDocumentURL];
}

- (void)deleteDocument: (NSURL *)deleteDocumentURL  {
  
  dispatch_async (dispatch_get_global_queue
                  (DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{                 //25
                    NSFileCoordinator *fc = [[NSFileCoordinator alloc]
                                             initWithFilePresenter:nil];
                    [fc coordinateWritingItemAtURL:deleteDocumentURL
                                           options:NSFileCoordinatorWritingForDeleting error:nil
                                        byAccessor:^(NSURL *newURL) {
                                          NSFileManager *fm = [[NSFileManager alloc] init];
                                          [fm removeItemAtURL: newURL error:nil];
                                        }];
                  });
  
  [_documents removeObject: deleteDocumentURL];                            //26
  
}
  
#pragma mark - debugging

- (void)checkOnQuery {                                                     //27
  if (_query) {
    NSLog (@" %lu", (unsigned long)[_query resultCount]);
    NSLog (@" %lu", (unsigned long)[_query isStarted]);
    NSLog (@" %lu", (unsigned long)[_query isGathering]);
    NSLog (@" %lu", (unsigned long)[_query isStopped]);
    [_query startQuery];
  }
}

@end

